# coding=utf-8
"""Command line interface to add tasks to 2Doapp."""

import argparse
import sys
import subprocess

import twodolib
from . urlhelper import TwoDoTask

usage_description = """\
Program to create tasks in 2Do. The default behavior is to print the generated
URL to stdout. Please use the '-e' or '--execute'' option, if you want to send 
the task directly to the 2DoApp.

Examples
========

Add a task due tomorrow:
task2do -d 1 "Dinner at 8pm."

Add a task with high priority:
task2do -p 3 "High priority task."
- or -
task2do --priority 3 "High priority task."

Add a task due today and repeated weekly:
task2do "change clothes" -d 0 --repeat 2

Add a task due at 6pm today
task2do "Watch EX_MACHINA" --due 0 --dueTime 18:00

Add a task due tomorrow, with tags, which is also starred and repeated monthly
task2do "Monthly subscription." --tags bill,payment -s --due 1 --repeat 4

Add a task with an url action (open a link)
task2do "Read help text" -a url:https://www.2doapp.com/
task2do "Read help text" --action url:https://www.2doapp.com/

Add a Subtask in list personal in project errands:
task2do "Buy milk." -l personal --project errands
"""

ACTION_HELP_TEXT = """action: Supports the following formats:

call:<number> - Add a Call action to call the specified number
message:<number> - Add a Message action to send a message to the specified 
                   number
mail:<email> - Add a Email action to send an email the specified email address
url:<url to visit> - Add a Browse action to visit the specified URL address
visit:<address> - Add a Visit action to visit the specified location
google:<search term> - Add a Google action to search the specified keyword

Enter the arguments after the colon without the angle brackets.

For more details: SEE https://www.2doapp.com/kb/article/url-schemes.html
"""


def parse_arguments(args):
    """Return Namespace with parsed command line arguments."""
    version = '%(prog)s {}'.format(twodolib.__version__)
    p = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        prog='task2do',
        description=usage_description,
    )
    p.add_argument('task', help='Title of the task.')
    p.add_argument('-a', '--action', help=ACTION_HELP_TEXT)
    p.add_argument('-d', '--due', default=None,
                   help='Due date. Supports two formats: YYYY-MM-DD - Sets '
                        'the date on default due time (based on your '
                        'settings), unless due time is specified separately '
                        'or ignoreDefaults (-i) is given. OR: Number '
                        'of days due from today. Which means: 0 = '
                        'today, 1 = tomorrow and so on)')
    p.add_argument('--dueTime', default=None,
                   help='Due time. Supports 24h format HH:MM.')
    p.add_argument('-e', '--execute', action='store_true',
                   help='Actually add the task instead of only '
                        'printing the URL to stdout.')
    p.add_argument('-f', '--forParentName', metavar='FOR_PARENTNAME',
                   dest='for_parent_name',
                   help='Title of an existing project or checklist to save the '
                        'new task there as a subtask. Also requires the '
                        'parent\'s task list.',
                   default=None)
    p.add_argument('-i', '--ignoreDefaults', action='store_true',
                   default=False,
                   help='Ignore default date / time settings of 2DoApp.')
    p.add_argument('-l', '--list', metavar='FOR_LIST', dest='for_list',
                   help='Name of an existing list in 2DoApp, '
                        'case-insensitive. If missing, the default list or the '
                        'currently visible list on screen is used.',
                   default=None)
    p.add_argument('-n', '--note', help='Notes for the task',
                   default=None)
    p.add_argument('-p', '--priority', choices=['0', '1', '2', '3'],
                   help='priority: 0 (none), 1 (low), 2 (medium), 3 (high)',
                   default='0')
    p.add_argument('--project', metavar='IN_PROJECT', dest='in_project',
                   help='Name of an existing project in 2DoApp, into which the'
                        ' task will be pasted. So you can create subtasks.',
                   default=None)
    p.add_argument('--repeat', default=None, choices=['1', '2', '3', '4'],
                   help='Repeat task: 1 (daily), 2 (weekly), 3 (bi-weekly), '
                        '4 (monthly))')
    p.add_argument('-s', '--starred', help='Mark task as starred.',
                   action='store_true', default=False)
    p.add_argument('--start', default=None,
                   help='Start date and time. Supports the format: '
                        '"YYYY-MM-DD HH:MM" - Sets the start date to the date '
                        'and time specified - OR - Any number with 0 = today, '
                        '1 = tomorrow and so on)')
    p.add_argument('-t', '--type', choices=['0', '1', '2'], dest='task_type',
                   help='Type of task to create. The following options are '
                        'supported: 0 - Task (default), 1 - Project, '
                        '2 - Checklist', default='0')
    p.add_argument('--tags', default=None,
                   help='Comma separated list of tags to assign to the task')
    p.add_argument('--taskid', default=None, action='store_true',
                   help='Prints taskid, needs the task title and the list.')
    p.add_argument('-v', '--version', action='version', version=version)
    return p.parse_args(args)


def main(arguments=None):  # use arguments to allow testing.
    """Create a task in 2DoApp."""
    if arguments is None:
        arguments = sys.argv[1:]
    args = parse_arguments(arguments)
    t = TwoDoTask(**vars(args))
    if args.execute:
        subprocess.call(['open', t.url()])
    elif args.taskid:
        if t.for_list:
            taskid = t.get_taskid()
            if taskid:
                print(taskid)
            else:
                print("No task found!")
        else:
            print('Please also provide the list '
                  'of the task you are looking for!')
    else:
        print(t.url())


if __name__ == '__main__':  # pragma: no cover
    main(sys.argv[1:])



# -*- coding: utf-8 -*-

"""Tests for `twodolib` module."""
import unittest
from urllib.parse import quote

import twodolib
from twodolib import TwoDoTask, pbcopy, pbpaste


class TestShowUrls(unittest.TestCase):
    """Test the 4 urls to show several lists in 2Do App."""

    def test_showall_url(self):
        """URL showall is correct."""
        expected_url = 'twodo://x-callback-url/showAll'
        self.assertEqual(twodolib.showall_url, expected_url)

    def test_showtoday_url(self):
        """URL showtoday is correct."""
        expected_url = 'twodo://x-callback-url/showToday'
        self.assertEqual(twodolib.showtoday_url, expected_url)

    def test_showstarred_url(self):
        """URL showstarred is correct."""
        expected_url = 'twodo://x-callback-url/showStarred'
        self.assertEqual(twodolib.showstarred_url, expected_url)

    def test_showscheduled_url(self):
        """URL showscheduled is correct."""
        expected_url = 'twodo://x-callback-url/showScheduled'
        self.assertEqual(twodolib.showscheduled_url, expected_url)


class TestTwoDoTaskClass(unittest.TestCase):
    """Test the creation and attributes of a TwoDoTask object."""

    def test_simple_task_with_title_only(self):
        """Create a simple task only by title."""
        title = 'A simple task by title'
        task = TwoDoTask(title)
        self.assertEqual(task.task, title)

    def test_default_values_of_simple_task(self):
        """Create a task by title and check the presence of attributes."""
        task = TwoDoTask('Save the world.')
        self.assertEqual(task.task, 'Save the world.')
        self.assertEqual(task.type, TwoDoTask.TASK_TYPE)
        self.assertIsNone(task.for_list)
        self.assertIsNone(task.forParentName)
        self.assertIsNone(task.note)
        self.assertEqual(task.priority, '0')
        self.assertEqual(task.starred, '0')
        self.assertIsNone(task.tags)
        self.assertIsNone(task.due)
        self.assertIsNone(task.dueTime)
        self.assertIsNone(task.start)
        self.assertIsNone(task.repeat)
        self.assertIsNone(task.action)
        self.assertEqual(task.ignoreDefaults, '0')


class TestMiscFunctionality(unittest.TestCase):
    """Test miscellaneous functions. """

    def test_method_get_taskid_url_exists(self):
        """Call the method to check if it is there."""
        task_title = 'Test title of the task.'
        task_list = 'errands'
        task = TwoDoTask(task_title, for_list=task_list)
        self.assertIsNotNone(task.get_taskid_url())

    def test_get_taskid_url_is_correct(self):
        """Test retrieval of a task id url."""
        task_title = 'Testtask'
        task_list = 'mylist'
        task = TwoDoTask(task_title, for_list=task_list)
        expected_url = ('twodo://x-callback-url/getTaskID?task=Testtask&'
                        'forList=mylist&saveInClipboard=1')
        self.assertEqual(task.get_taskid_url(), expected_url)

    def test_get_non_existing_taskid_returns_false(self):
        """Test retrieval of a non existing task id gives 'False'."""
        task_title = 'This task won\'t exist. AxTT6%gB77U Testtask'
        task_list = 'this list neither, so we do not get an ID!'
        task = TwoDoTask(task_title, for_list=task_list)
        self.assertEqual(task.get_taskid(), False)

    def test_copy_and_paste(self):
        """Copy text to clipboard and retrieve it."""
        text = "Hello MacOS clipboard!"
        message = ""
        pbcopy(text)
        message = pbpaste()
        self.assertEqual(text, message)


class TestGeneratedUrlsOfTwoDoTask(unittest.TestCase):
    """Test several URLs, which are generated by a TwoDoTask object."""

    def test_simple_task_url_is_correct(self):
        """The URL for a simple task is correct."""
        task_title = 'Test title of the task.'
        quoted_title = quote(task_title)
        expected_url = 'twodo://x-callback-url/add?task=' + quoted_title
        task = TwoDoTask(task_title)
        self.assertEqual(task.url(), expected_url)

    def test_project_task_url_is_correct(self):
        """The URL for a project task is correct."""
        task_title = 'Test title of the task.'
        quoted_title = quote(task_title)
        expected_url = 'twodo://x-callback-url/add?task=' + quoted_title
        expected_url += '&type=1'
        task = TwoDoTask(task_title, task_type='1')
        self.assertEqual(task.url(), expected_url)

    def test_task_for_a_list_url_is_correct(self):
        """The URL for a task for a list is correct."""
        task_title = 'Test title of the task.'
        quoted_title = quote(task_title)
        task_list = 'urgent errands'
        quoted_list = quote(task_list)
        expected_url = 'twodo://x-callback-url/add?task=' + quoted_title
        expected_url += '&forlist={}'.format(quoted_list)
        task = TwoDoTask(task_title, for_list=task_list)
        self.assertEqual(task.url(), expected_url)

    def test_task_with_a_note_url_is_correct(self):
        """The URL for a task with a note is correct."""
        task_title = 'Test title of the task.'
        quoted_title = quote(task_title)
        task_note = "A little note to this important task."
        quoted_note = quote(task_note)
        expected_url = 'twodo://x-callback-url/add?task=' + quoted_title
        expected_url += '&note={}'.format(quoted_note)
        task = TwoDoTask(task_title, note=task_note)
        self.assertEqual(task.url(), expected_url)

    def test_task_with_prio_url_is_correct(self):
        """The URL for a task with a priority is correct."""
        task_title = 'Test title of the task.'
        quoted_title = quote(task_title)
        expected_url = 'twodo://x-callback-url/add?task=' + quoted_title
        expected_url += '&priority=3'
        task = TwoDoTask(task_title, priority=3)
        self.assertEqual(task.url(), expected_url)

    def test_starred_task_url_is_correct_boolean(self):
        """The URL for a starred task is correct."""
        task_title = 'Test title of the task.'
        quoted_title = quote(task_title)
        expected_url = 'twodo://x-callback-url/add?task=' + quoted_title
        expected_url += '&starred=1'
        task = TwoDoTask(task_title, starred=True)
        self.assertEqual(task.url(), expected_url)

    def test_starred_task_url_is_correct_one(self):
        """The URL for a starred task is correct."""
        task_title = 'Test title of the task.'
        quoted_title = quote(task_title)
        expected_url = 'twodo://x-callback-url/add?task=' + quoted_title
        expected_url += '&starred=1'
        task = TwoDoTask(task_title, starred=True)
        self.assertEqual(task.url(), expected_url)

    def test_task_with_tags_url_is_correct(self):
        """The URL for a task with some tags is correct."""
        task_title = 'Test title of the task.'
        quoted_title = quote(task_title)
        task_tags = "business,important,customer"
        quoted_tags = quote(task_tags)
        expected_url = 'twodo://x-callback-url/add?task=' + quoted_title
        expected_url += '&tags={}'.format(quoted_tags)
        task = TwoDoTask(task_title, tags=task_tags)
        self.assertEqual(task.url(), expected_url)

    def test_task_with_iso_duedate_url_is_correct(self):
        """The URL for a task with a iso formatted due date is correct."""
        task_title = 'Test title of the task.'
        quoted_title = quote(task_title)
        task_due = "2015-10-02"
        quoted_due = quote(task_due)
        expected_url = 'twodo://x-callback-url/add?task=' + quoted_title
        expected_url += '&due={}'.format(quoted_due)
        task = TwoDoTask(task_title, due=task_due)
        self.assertEqual(task.url(), expected_url)

    def test_task_with_rel_duedate_url_is_correct(self):
        """The URL for a task with a relative due date is correct."""
        task_title = 'Test title of the task.'
        quoted_title = quote(task_title)
        expected_url = 'twodo://x-callback-url/add?task=' + quoted_title
        expected_url += '&due=7'
        task = TwoDoTask(task_title, due='7')
        self.assertEqual(task.url(), expected_url)

    def test_task_with_duetime_url_is_correct(self):
        """The URL for a task with a due time is correct."""
        task_title = 'Test title of the task.'
        quoted_title = quote(task_title)
        task_duetime = "12:00"
        quoted_duetime = quote(task_duetime)
        expected_url = 'twodo://x-callback-url/add?task=' + quoted_title
        expected_url += '&dueTime={}'.format(quoted_duetime)
        task = TwoDoTask(task_title, dueTime=task_duetime)
        self.assertEqual(task.url(), expected_url)

    def test_task_with_start_time_url_is_correct(self):
        """The URL for a task with a start time ist correct."""
        task_title = 'Test title of the task.'
        quoted_title = quote(task_title)
        task_start = "2015-10-01 12:00"
        quoted_start = quote(task_start)
        expected_url = 'twodo://x-callback-url/add?task=' + quoted_title
        expected_url += '&start={}'.format(quoted_start)
        task = TwoDoTask(task_title, start=task_start)
        self.assertEqual(task.url(), expected_url)

    def test_task_with_rel_start_time_url_is_correct(self):
        """The URL for a task with a relative start date ist correct."""
        task_title = 'Test title of the task.'
        quoted_title = quote(task_title)
        task_start = "14"
        quoted_start = quote(task_start)
        expected_url = 'twodo://x-callback-url/add?task=' + quoted_title
        expected_url += '&start={}'.format(quoted_start)
        task = TwoDoTask(task_title, start=task_start)
        self.assertEqual(task.url(), expected_url)

    def test_task_with_daily_repeat_url_is_correct(self):
        """The URL for a task with daily repetition is correct."""
        task_title = 'Test title of the task.'
        quoted_title = quote(task_title)
        expected_url = 'twodo://x-callback-url/add?task=' + quoted_title
        expected_url += '&repeat={}'.format(TwoDoTask.DAILY)
        task = TwoDoTask(task_title, repeat='1')
        self.assertEqual(task.url(), expected_url)

    def test_task_add_to_project_is_correct(self):
        """The URL for pasting a task to a project is correct."""
        task_title = 'Buy milk.'
        in_list = "personal"
        quoted_in_list = quote(in_list)
        in_project = "errands"
        quoted_in_project = quote(in_project)
        quoted_title = quote(task_title)
        expected_url = 'twodo://x-callback-url/paste?text=' + quoted_title
        expected_url += '&inProject=' + quoted_in_project
        expected_url += '&forList=' + quoted_in_list
        task = TwoDoTask(task_title, for_list=in_list, in_project=in_project)
        self.assertEqual(task.url(), expected_url)


class TestTwoDoTaskValidation(unittest.TestCase):
    """Test the validation of arguments, when creating a TwoDoTask object."""

    def test_task_with_title_length_of_zero_raises_valueerror(self):
        """Create a simple task without title raises ValueError."""
        title = ''
        self.assertRaises(ValueError, TwoDoTask, title)

    def test_task_with_title_is_none_raises_valueerror(self):
        """Create a simple task with None title raises ValueError."""
        title = None
        self.assertRaises(ValueError, TwoDoTask, title)

    def test_wrong_type_raises_valueerror(self):
        """Create Task with Type invalid Type raises ValueError."""
        self.assertRaises(ValueError, TwoDoTask, 'TestTask', task_type='4')

    def test_wrong_priority_raises_valueerror(self):
        """Create Task with Type invalid priority raises ValueError."""
        self.assertRaises(ValueError, TwoDoTask, 'TestTask', priority='4')

    def test_project_without_list_raises_valueerror(self):
        """A task can only be pasted into a project if a list is given."""
        task = TwoDoTask('TestTask', in_project='X')
        self.assertRaises(ValueError, task.url)

    def test_starred_true_returns_1_value(self):
        """A True starred will be represented as '1'."""
        task = TwoDoTask('TestTask', starred=True)
        self.assertEqual(task.starred, '1')

    def test_starred_1_returns_1_value(self):
        """A 1 starred will be represented as '1'."""
        task = TwoDoTask('TestTask', starred=1)
        self.assertEqual(task.starred, '1')

    def test_starred_str_1_returns_1_value(self):
        """A '1' starred will be represented as '1'."""
        task = TwoDoTask('TestTask', starred='1')
        self.assertEqual(task.starred, '1')

    def test_starred_false_returns_0_value(self):
        """A False starred will be represented as '0'."""
        task = TwoDoTask('TestTask', starred=False)
        self.assertEqual(task.starred, '0')

    def test_starred_0_returns_0_value(self):
        """A 0 starred will be represented as '0'."""
        task = TwoDoTask('TestTask', starred=0)
        self.assertEqual(task.starred, '0')

    def test_starred_str_0_returns_0_value(self):
        """A '0' starred will be represented as '0'."""
        task = TwoDoTask('TestTask', starred='0')
        self.assertEqual(task.starred, '0')

    def test_ignoredefaults_true_returns_1_value(self):
        """A True ignoreDefaults will be represented as '1'."""
        task = TwoDoTask('TestTask', ignoreDefaults=True)
        self.assertEqual(task.ignoreDefaults, '1')

    def test_ignoredefaults_1_returns_1_value(self):
        """A 1 ignoreDefaults will be represented as '1'."""
        task = TwoDoTask('TestTask', ignoreDefaults=1)
        self.assertEqual(task.ignoreDefaults, '1')

    def test_ignoredefaults_str_1_returns_1_value(self):
        """A '1' ignoreDefaults will be represented as '1'."""
        task = TwoDoTask('TestTask', ignoreDefaults='1')
        self.assertEqual(task.ignoreDefaults, '1')

    def test_ignoredefaults_false_returns_0_value(self):
        """A False ignoreDefaults will be represented as '0'."""
        task = TwoDoTask('TestTask', ignoreDefaults=False)
        self.assertEqual(task.ignoreDefaults, '0')

    def test_ignoredefaults_0_returns_0_value(self):
        """A 0 ignoreDefaults will be represented as '0'."""
        task = TwoDoTask('TestTask', ignoreDefaults=0)
        self.assertEqual(task.ignoreDefaults, '0')

    def test_ignoredefaults_str_0_returns_0_value(self):
        """A '0' ignoreDefaults will be represented as '0'."""
        task = TwoDoTask('TestTask', ignoreDefaults='0')
        self.assertEqual(task.ignoreDefaults, '0')

    def test_correct_due_date_format_will_be_accepted(self):
        """A correct date will be stored in the task."""
        task = TwoDoTask('TestTask', due='2015-10-15')
        self.assertEqual(task.due, '2015-10-15')

    def test_correct_due_format_will_be_accepted(self):
        """A correct date will be stored in the task."""
        task = TwoDoTask('TestTask', due='14')
        self.assertEqual(task.due, '14')

    def test_correct_due_int_format_will_be_accepted(self):
        """A correct date will be stored in the task."""
        task = TwoDoTask('TestTask', due=14)
        self.assertEqual(task.due, '14')

    def test_wrong_due_format_raises_valueerror(self):
        """A wrong date format in due date raises ValueError."""
        wrong_date = '2015-XX-10'
        self.assertRaises(ValueError, TwoDoTask, 'TestTask', due=wrong_date)
        wrong_date = '15.10.2015'
        self.assertRaises(ValueError, TwoDoTask, 'TestTask', due=wrong_date)
        wrong_date = '2015......'
        self.assertRaises(ValueError, TwoDoTask, 'TestTask', due=wrong_date)

    def test_task_with_wrong_duetime_raises_valueerror(self):
        """Wrong formatted due time raises ValueError."""
        task_title = 'Test title of the task.'
        task_duetime = "8pm"
        self.assertRaises(ValueError, TwoDoTask, task_title,
                          dueTime=task_duetime)

    def test_task_with_correct_duetime_is_accepted(self):
        """Wrong formatted due time raises ValueError."""
        task_title = 'Test title of the task.'
        task_duetime = "8:00"
        t = TwoDoTask(task=task_title, dueTime=task_duetime)
        self.assertEqual(t.dueTime, '8:00')

    def test_wrong_repetition_raises_valueerror(self):
        """Wrong values for repetition raise ValueError."""
        self.assertRaises(ValueError, TwoDoTask, 'TestTask', repeat=0)
        self.assertRaises(ValueError, TwoDoTask, 'TestTask', repeat='5')



# -*- coding: utf-8 -*-

"""Tests for `twodolib.cli` module."""
import unittest
import subprocess
from twodolib import TwoDoTask
from twodolib import cli


class TestCliParseArguments(unittest.TestCase):
    """Test the command line interface and argument parsing."""

    def test_default_task_is_task_type(self):
        """Default task type should be 'task'."""
        parsed = cli.parse_arguments(['TestTask'])
        self.assertEqual(parsed.task_type, TwoDoTask.TASK_TYPE)

    def test_set_task_type_project(self):
        """A task can be a 'project' type: -t 1."""
        args = "TestTask -t 1".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.task_type, TwoDoTask.PROJECT_TYPE)

    def test_set_task_type_project_long_option(self):
        """A task can be a 'project' type: --type 1."""
        args = "TestTask --type 1".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.task_type, TwoDoTask.PROJECT_TYPE)

    def test_set_task_type_checklist(self):
        """A task can be a 'checklist' type: -t 2."""
        args = "TestTask -t 2".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.task_type, TwoDoTask.CHECKLIST_TYPE)

    def test_set_task_type_checklist_long_option(self):
        """A task can be a 'checklist' type: --type 2."""
        args = "TestTask --type 2".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.task_type, TwoDoTask.CHECKLIST_TYPE)

    def test_set_list_name(self):
        """A task can belong to a list: -l listname."""
        args = "TestTask -l business".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.for_list, 'business')

    def test_set_project_name(self):
        """A task can be a subtask in a project."""
        args = "TestTask -l business --project webpage".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.for_list, 'business')
        self.assertEqual(parsed.in_project, 'webpage')

    def test_set_list_name_long_option(self):
        """A task can belong to a list: --list listname."""
        args = "TestTask --list business".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.for_list, 'business')

    def test_set_note_of_task(self):
        """A task can have a note: -n "some notes to the task ..."."""
        args = [
            'TestTask',
            '-n',
            'This is a note for the task.'
        ]
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.note, 'This is a note for the task.')

    def test_set_note_of_task_long_option(self):
        """A task can have a note: --note "some notes to the task ..."."""
        args = [
            'TestTask',
            '--note',
            'This is a note for the task.'
        ]
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.note, 'This is a note for the task.')

    def test_set_priority(self):
        """Priority of a task can be set: -p 2."""
        args = "TestTask -p 2".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.priority, '2')

    def test_set_priority_long_option(self):
        """Priority of a task can be set: --priority 2."""
        args = "TestTask --priority 2".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.priority, '2')

    def test_not_starring_task_is_default(self):
        """The default task is not starred."""
        args = ["TestTask"]
        parsed = cli.parse_arguments(args)
        self.assertFalse(parsed.starred)

    def test_starr_task(self):
        """Starr a task with: -s."""
        args = "TestTask -s".split()
        parsed = cli.parse_arguments(args)
        self.assertTrue(parsed.starred)

    def test_starr_task_long_option(self):
        """Starr a task with: --starred."""
        args = "TestTask --starred".split()
        parsed = cli.parse_arguments(args)
        self.assertTrue(parsed.starred)

    def test_set_tags_in_task(self):
        """A Task can have tags."""
        args = "TestTask --tags business,customer,important".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.tags, "business,customer,important")

    def test_get_taskid(self):
        """Print the task id"""
        args = "TestTask --taskid -l mylist".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.taskid, True)

    def test_set_due_date_isoformat(self):
        """Set the tasks due date: -d YYYY-MM-DD."""
        args = "TestTask -d 2015-09-10".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.due, "2015-09-10")

    def test_use_parenttask(self):
        """Set the parent task of a subtask"""
        args = "SubTask -f Parenttask -l mylist".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.for_parent_name, "Parenttask")

    def test_set_due_date_isoformat_long_option(self):
        """Set the tasks due date: --due YYYY-MM-DD."""
        args = "TestTask --due 2015-09-10".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.due, "2015-09-10")

    def test_set_due_date_n_format(self):
        """Set the tasks due date: -d 2."""
        args = "TestTask -d 2".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.due, "2")

    def test_set_due_date_n_format_long_option(self):
        """Set the tasks due date: --due 14."""
        args = "TestTask --due 14".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.due, "14")

    def test_set_duetime(self):
        """Set the tasks due time: --dueTime '2015-09-10 12:00'."""
        args = "TestTask --dueTime".split()
        args.append('2015-09-10 12:00')
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.dueTime, "2015-09-10 12:00")

    def test_set_start_isoformat(self):
        """Set the tasks start time: --start '2015-09-10 12:00'."""
        args = "TestTask --start".split()
        args.append('2015-09-10 12:00')
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.start, "2015-09-10 12:00")

    def test_set_start_n_format(self):
        """Set the tasks start time: --start 7."""
        args = "TestTask --start 7".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.start, "7")

    def test_repeat_task(self):
        """Create a repeating task: --repeat 2."""
        args = "TestTask --repeat 2".split()
        parsed = cli.parse_arguments(args)
        self.assertEqual(parsed.repeat, "2")

    def test_ignoredefaults_default_to_false(self):
        """Do not ignore defaults."""
        parsed = cli.parse_arguments(['TestTask'])
        self.assertFalse(parsed.ignoreDefaults)

    def test_set_ignoredefaults(self):
        """Ignore defaults: -i."""
        args = "TestTask -i".split()
        parsed = cli.parse_arguments(args)
        self.assertTrue(parsed.ignoreDefaults)

    def test_set_ignoredefaults_long_option(self):
        """Ignore defaults: --ignoerDefaults."""
        args = "TestTask --ignoreDefaults".split()
        parsed = cli.parse_arguments(args)
        self.assertTrue(parsed.ignoreDefaults)

    def test_missing_args_raise_system_exit(self):
        """Raise SystemExit, if args are missing."""
        with self.assertRaises(SystemExit):
            cli.main([])


class TestCliGeneratesCorrectTwoDoTaskObject(unittest.TestCase):
    """Command line args should create correct task object."""

    def test_create_simple_task(self):
        """Create a task with a title."""
        parsed = cli.parse_arguments(['TestTask'])
        task = TwoDoTask(**vars(parsed))
        self.assertEqual(task.task, 'TestTask')

    def test_create_simple_task_has_correct_defaults(self):
        """Create a task with correct defaults."""
        parsed = cli.parse_arguments(['TestTask'])
        task = TwoDoTask(**vars(parsed))
        self.assertEqual(task.task, 'TestTask')
        self.assertEqual(task.type, TwoDoTask.TASK_TYPE)
        self.assertIsNone(task.for_list)
        self.assertIsNone(task.forParentName)
        self.assertIsNone(task.note)
        self.assertEqual(task.priority, '0')
        self.assertEqual(task.starred, '0')
        self.assertIsNone(task.tags)
        self.assertIsNone(task.due)
        self.assertIsNone(task.dueTime)
        self.assertIsNone(task.start)
        self.assertIsNone(task.repeat)
        self.assertIsNone(task.action)
        self.assertEqual(task.ignoreDefaults, '0')

    def test_task_has_repeat_and_priority(self):
        """Create a task with weekly repetition and high priority."""
        args = "TestTask --repeat 1 --priority 2".split()
        parsed = cli.parse_arguments(args)
        task = TwoDoTask(**vars(parsed))
        self.assertEqual(task.task, 'TestTask')
        self.assertEqual(task.type, TwoDoTask.TASK_TYPE)
        self.assertIsNone(task.for_list)
        self.assertIsNone(task.forParentName)
        self.assertIsNone(task.note)
        self.assertEqual(task.priority, '2')
        self.assertEqual(task.starred, '0')
        self.assertIsNone(task.tags)
        self.assertIsNone(task.due)
        self.assertIsNone(task.dueTime)
        self.assertIsNone(task.start)
        self.assertEqual(task.repeat, '1')
        self.assertIsNone(task.action)
        self.assertEqual(task.ignoreDefaults, '0')

    def test_task_is_starred(self):
        """Create a starred task."""
        args = "TestTask -s".split()
        parsed = cli.parse_arguments(args)
        task = TwoDoTask(**vars(parsed))
        self.assertEqual(task.task, 'TestTask')
        self.assertEqual(task.starred, '1')

    def test_set_task_to_ignore_defaults(self):
        """Create a task, which ignores date and time defaults."""
        args = "TestTask -i".split()
        parsed = cli.parse_arguments(args)
        task = TwoDoTask(**vars(parsed))
        self.assertEqual(task.task, 'TestTask')
        self.assertEqual(task.ignoreDefaults, '1')

    def test_call_of_cli_generates_output(self):
        """Call command generates no output."""
        msg = subprocess.check_output(['python', '-m', 'twodolib.cli', 'test'])
        self.assertGreater(len(msg), 0)

    def test_task_gets_action_long(self):
        """Create a task with an action."""
        args = "TestTask --action url:https://www.2doapp.com".split()
        parsed = cli.parse_arguments(args)
        task = TwoDoTask(**vars(parsed))
        self.assertEqual(task.task, 'TestTask')
        self.assertEqual(task.type, TwoDoTask.TASK_TYPE)
        self.assertIsNone(task.for_list)
        self.assertIsNone(task.forParentName)
        self.assertIsNone(task.note)
        self.assertEqual(task.priority, '0')
        self.assertEqual(task.starred, '0')
        self.assertIsNone(task.tags)
        self.assertIsNone(task.due)
        self.assertIsNone(task.dueTime)
        self.assertIsNone(task.start)
        self.assertIsNone(task.repeat)
        self.assertIsNotNone(task.action)
        self.assertEqual(task.ignoreDefaults, '0')

    def test_task_gets_action_short(self):
        """Create a task with an action."""
        args = "TestTask -a url:https://www.2doapp.com".split()
        parsed = cli.parse_arguments(args)
        task = TwoDoTask(**vars(parsed))
        self.assertEqual(task.task, 'TestTask')
        self.assertEqual(task.type, TwoDoTask.TASK_TYPE)
        self.assertIsNone(task.for_list)
        self.assertIsNone(task.forParentName)
        self.assertIsNone(task.note)
        self.assertEqual(task.priority, '0')
        self.assertEqual(task.starred, '0')
        self.assertIsNone(task.tags)
        self.assertIsNone(task.due)
        self.assertIsNone(task.dueTime)
        self.assertIsNone(task.start)
        self.assertIsNone(task.repeat)
        self.assertIsNotNone(task.action)
        self.assertEqual(task.ignoreDefaults, '0')
